@@ -27,7 +27,7 @@ class DataNode(DatasetManager):
             description = f"data {name}"
             
         name = f'{name}.parquet'
-        replace = self._check_fileExists(directory, name) if replace == None else replace
+        replace = self._check_fileExists(directory, name) if replace is None else replace
         
         _res = requests.post(f'{self._discovery_api}/file/', headers=self._jwt_header,
                                 json={
@@ -229,15 +229,18 @@ class DatabaseManagement(Base):
         
 
     
-    def write_sql_query(self, query_function=None, directory_id=None, table_id=None, pk_column=None, meta=None, name=None, desciption=None, *args, **kwargs):
+    def write_sql_query(self, df=None, query_function=None, directory_id=None, table_id=None, pk_column=None, 
+                        meta=None, name=None, desciption=None, replace=None, *args, **kwargs):
         '''Write sql_connection in pickle with checking permission
         
         Args:
         ...
         '''
+        if type(df) not in [pd.DataFrame, dd.DataFrame]:
+            raise Exception(f"Invalid type expect ddf(dask dataframe) or df(pandas dataframe), Please input `df=`, but got {type(df)}")
         
-        if query_function is None or inspect.isfunction(query_function)==False:
-            raise Exception(f"Expect `query`=<function> but got {type(query_function)}, please input `query`")
+        if query_function is None or (inspect.isfunction(query_function)==False and type(query_function)!=str):
+            raise Exception(f"Expect `query`=<function>/<str> but got {type(query_function)}, please input `query`")
         
         if directory_id == None or type(directory_id) != int:
             raise Exception(f"Please input data `directory`=<int>, but got {type(directory_id)} eg directory_id=0")
@@ -256,9 +259,12 @@ class DatabaseManagement(Base):
         
         if desciption == None:
             desciption = f"query of {name}"
+    
+        _replace = self._check_fileExists(directory_id, name) if replace is None else replace
         
-        _replace = self._check_fileExists(directory=directory_id, name=name)
-        
+        desired_order_list = [pk_column] + df.columns.tolist()
+        reordered_meta = {k: meta[k] for k in desired_order_list}
+
         r = requests.post(f"{self._base_discovery_api}/api/sql/query/", headers=self._jwt_header,
                           json={
                                 "directory": directory_id,
@@ -266,8 +272,8 @@ class DatabaseManagement(Base):
                                 "name": name,
                                 "description": desciption,
                                 "pk_column": pk_column,
-                                "query": inspect.getsource(query_function),
-                                "meta": meta,
+                                "query": inspect.getsource(query_function) if inspect.isfunction(query_function) else query_function,
+                                "meta": reordered_meta,
                                 "replace": _replace
                           }
         )
